@@ -1,6 +1,6 @@
 /* Provide a more complete sys/file.h.
 
-   Copyright (C) 2007-2011 Free Software Foundation, Inc.
+   Copyright (C) 2007-2012 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -13,12 +13,11 @@
    GNU Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 /* Written by Richard W.M. Jones.  */
 
-#ifndef _GL_SYS_FILE_H
+#ifndef _@GUARD_PREFIX@_SYS_FILE_H
 
 #if __GNUC__ >= 3
 @PRAGMA_SYSTEM_HEADER@
@@ -30,8 +29,8 @@
 # @INCLUDE_NEXT@ @NEXT_SYS_FILE_H@
 #endif
 
-#ifndef _GL_SYS_FILE_H
-#define _GL_SYS_FILE_H
+#ifndef _@GUARD_PREFIX@_SYS_FILE_H
+#define _@GUARD_PREFIX@_SYS_FILE_H
 
 #ifndef LOCK_SH
 /* Operations for the 'flock' call (same as Linux kernel constants).  */
@@ -60,5 +59,5 @@ _GL_WARN_ON_USE (flock, "flock is unportable - "
 #endif
 
 
-#endif /* _GL_SYS_FILE_H */
-#endif /* _GL_SYS_FILE_H */
+#endif /* _@GUARD_PREFIX@_SYS_FILE_H */
+#endif /* _@GUARD_PREFIX@_SYS_FILE_H */
