@@ -1,4 +1,4 @@
-/* Copyright (C) 1995,1996,1997,1998,2000,2001, 2003, 2004, 2009 Free Software
+/* Copyright (C) 1995,1996,1997,1998,2000,2001, 2003, 2004, 2009, 2010 Free Software
  * Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
@@ -127,7 +127,7 @@ SCM_DEFINE (scm_system_star, "system*", 0, 0, 1,
       execargv = scm_i_allocate_string_pointers (args);
 
       /* make sure the child can't kill us (as per normal system call) */
-      sig_ign = scm_from_long ((unsigned long) SIG_IGN);
+      sig_ign = scm_from_ulong ((unsigned long) SIG_IGN);
       sigint = scm_from_int (SIGINT);
       sigquit = scm_from_int (SIGQUIT);
       oldint = scm_sigaction (sigint, sig_ign, SCM_UNDEFINED);
@@ -169,7 +169,7 @@ SCM_DEFINE (scm_system_star, "system*", 0, 0, 1,
 
 SCM_DEFINE (scm_getenv, "getenv", 1, 0, 0, 
             (SCM nam),
-	    "Looks up the string @var{name} in the current environment.  The return\n"
+	    "Looks up the string @var{nam} in the current environment.  The return\n"
 	    "value is @code{#f} unless a string of the form @code{NAME=VALUE} is\n"
 	    "found, in which case the string @code{VALUE} is returned.")
 #define FUNC_NAME s_scm_getenv
