@@ -1,4 +1,4 @@
-/* Copyright (C) 1996,1997,1998,1999,2000,2001, 2003, 2004, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+/* Copyright (C) 1996,1997,1998,1999,2000,2001, 2003, 2004, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -54,7 +54,6 @@
 static SCM required_vtable_fields = SCM_BOOL_F;
 static SCM required_applicable_fields = SCM_BOOL_F;
 static SCM required_applicable_with_setter_fields = SCM_BOOL_F;
-SCM scm_struct_table = SCM_BOOL_F;
 SCM scm_applicable_struct_vtable_vtable;
 SCM scm_applicable_struct_with_setter_vtable_vtable;
 SCM scm_standard_vtable_vtable;
@@ -411,7 +410,7 @@ SCM_DEFINE (scm_struct_vtable_p, "struct-vtable?", 1, 0, 0,
 
 /* Finalization: invoke the finalizer of the struct pointed to by PTR.  */
 static void
-struct_finalizer_trampoline (GC_PTR ptr, GC_PTR unused_data)
+struct_finalizer_trampoline (void *ptr, void *unused_data)
 {
   SCM obj = PTR2SCM (ptr);
   scm_t_struct_finalize finalize = SCM_STRUCT_FINALIZER (obj);
@@ -444,16 +443,8 @@ scm_i_alloc_struct (scm_t_bits *vtable_data, int n_words)
 
   /* vtable_data can be null when making a vtable vtable */
   if (vtable_data && vtable_data[scm_vtable_index_instance_finalize])
-    {
-      /* Register a finalizer for the newly created instance.  */
-      GC_finalization_proc prev_finalizer;
-      GC_PTR prev_finalizer_data;
-      GC_REGISTER_FINALIZER_NO_ORDER (SCM2PTR (ret),
-				      struct_finalizer_trampoline,
-				      NULL,
-				      &prev_finalizer,
-				      &prev_finalizer_data);
-    }
+    /* Register a finalizer for the newly created instance.  */
+    scm_i_set_finalizer (SCM2PTR (ret), struct_finalizer_trampoline, NULL);
 
   return ret;
 }
@@ -528,9 +519,9 @@ scm_c_make_struct (SCM vtable, size_t n_tail, size_t n_init, scm_t_bits init, ..
 SCM_DEFINE (scm_make_struct, "make-struct", 2, 0, 1, 
             (SCM vtable, SCM tail_array_size, SCM init),
 	    "Create a new structure.\n\n"
-	    "@var{type} must be a vtable structure (@pxref{Vtables}).\n\n"
-	    "@var{tail-elts} must be a non-negative integer.  If the layout\n"
-	    "specification indicated by @var{type} includes a tail-array,\n"
+	    "@var{vtable} must be a vtable structure (@pxref{Vtables}).\n\n"
+	    "@var{tail_array_size} must be a non-negative integer.  If the layout\n"
+	    "specification indicated by @var{vtable} includes a tail-array,\n"
 	    "this is the number of elements allocated to that array.\n\n"
 	    "The @var{init1}, @dots{} are optional arguments describing how\n"
 	    "successive fields of the structure should be initialized.  Only fields\n"
@@ -577,7 +568,7 @@ SCM_DEFINE (scm_make_vtable_vtable, "make-vtable-vtable", 2, 0, 1,
 	    "@var{user-fields} is a string describing user defined fields of the\n"
 	    "vtable beginning at index @code{vtable-offset-user}\n"
 	    "(see @code{make-struct-layout}).\n\n"
-	    "@var{tail-size} specifies the size of the tail-array (if any) of\n"
+	    "@var{tail_array_size} specifies the size of the tail-array (if any) of\n"
 	    "this vtable.\n\n"
 	    "@var{init1}, @dots{} are the optional initializers for the fields of\n"
 	    "the vtable.\n\n"
@@ -739,10 +730,15 @@ scm_i_struct_equalp (SCM s1, SCM s2)
 
 SCM_DEFINE (scm_struct_ref, "struct-ref", 2, 0, 0,
             (SCM handle, SCM pos),
-	    "Access the @var{n}th field of @var{struct}.\n\n"
-	    "If the field is of type 'p', then it can be set to an arbitrary value.\n\n"
-	    "If the field is of type 'u', then it can only be set to a non-negative\n"
-	    "integer value small enough to fit in one machine word.")
+	    "Access the @var{pos}th field of struct associated with\n"
+	    "@var{handle}.\n"
+	    "\n"
+	    "If the field is of type 'p', then it can be set to an arbitrary\n"
+	    "value.\n"
+	    "\n"
+	    "If the field is of type 'u', then it can only be set to a\n"
+	    "non-negative integer value small enough to fit in one machine\n"
+	    "word.")
 #define FUNC_NAME s_scm_struct_ref
 {
   SCM vtable, answer = SCM_UNDEFINED;
@@ -911,7 +907,8 @@ SCM_DEFINE (scm_struct_set_x, "struct-set!", 3, 0, 0,
 
 SCM_DEFINE (scm_struct_vtable, "struct-vtable", 1, 0, 0, 
             (SCM handle),
-	    "Return the vtable structure that describes the type of @var{struct}.")
+	    "Return the vtable structure that describes the type of struct\n"
+	    "associated with @var{handle}.")
 #define FUNC_NAME s_scm_struct_vtable
 {
   SCM_VALIDATE_STRUCT (1, handle);
@@ -920,17 +917,6 @@ SCM_DEFINE (scm_struct_vtable, "struct-vtable", 1, 0, 0,
 #undef FUNC_NAME
 
 
-SCM_DEFINE (scm_struct_vtable_tag, "struct-vtable-tag", 1, 0, 0, 
-            (SCM handle),
-	    "Return the vtable tag of the structure @var{handle}.")
-#define FUNC_NAME s_scm_struct_vtable_tag
-{
-  SCM_VALIDATE_VTABLE (1, handle);
-  return scm_from_unsigned_integer
-    (((scm_t_bits)SCM_STRUCT_DATA (handle)) >> 3);
-}
-#undef FUNC_NAME
-
 /* {Associating names and classes with vtables}
  *
  * The name of a vtable should probably be stored as a slot.  This is
@@ -946,27 +932,13 @@ scm_struct_ihashq (SCM obj, unsigned long n, void *closure)
   return SCM_UNPACK (obj) % n;
 }
 
-SCM
-scm_struct_create_handle (SCM obj)
-{
-  SCM handle = scm_hash_fn_create_handle_x (scm_struct_table,
-					    obj,
-					    SCM_BOOL_F,
-					    scm_struct_ihashq,
-					    (scm_t_assoc_fn) scm_sloppy_assq,
-					    0);
-  if (scm_is_false (SCM_CDR (handle)))
-    SCM_SETCDR (handle, scm_cons (SCM_BOOL_F, SCM_BOOL_F));
-  return handle;
-}
-
 SCM_DEFINE (scm_struct_vtable_name, "struct-vtable-name", 1, 0, 0, 
             (SCM vtable),
 	    "Return the name of the vtable @var{vtable}.")
 #define FUNC_NAME s_scm_struct_vtable_name
 {
   SCM_VALIDATE_VTABLE (1, vtable);
-  return SCM_STRUCT_TABLE_NAME (SCM_CDR (scm_struct_create_handle (vtable)));
+  return SCM_VTABLE_NAME (vtable);
 }
 #undef FUNC_NAME
 
@@ -977,8 +949,10 @@ SCM_DEFINE (scm_set_struct_vtable_name_x, "set-struct-vtable-name!", 2, 0, 0,
 {
   SCM_VALIDATE_VTABLE (1, vtable);
   SCM_VALIDATE_SYMBOL (2, name);
-  SCM_SET_STRUCT_TABLE_NAME (SCM_CDR (scm_struct_create_handle (vtable)),
-			     name);
+  SCM_SET_VTABLE_NAME (vtable, name);
+  /* FIXME: remove this, and implement proper struct classes instead.
+     (Vtables *are* classes.)  */
+  scm_i_define_class_for_vtable (vtable);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
@@ -1047,13 +1021,14 @@ scm_init_struct ()
      OBJ once OBJ has undergone class redefinition.  */
   GC_REGISTER_DISPLACEMENT (2 * sizeof (scm_t_bits));
 
-  scm_struct_table = scm_make_weak_key_hash_table (scm_from_int (31));
   required_vtable_fields = scm_from_locale_string (SCM_VTABLE_BASE_LAYOUT);
+  scm_c_define ("standard-vtable-fields", required_vtable_fields);
   required_applicable_fields = scm_from_locale_string (SCM_APPLICABLE_BASE_LAYOUT);
   required_applicable_with_setter_fields = scm_from_locale_string (SCM_APPLICABLE_WITH_SETTER_BASE_LAYOUT);
 
   scm_standard_vtable_vtable =
     scm_make_vtable_vtable (scm_nullstr, SCM_INUM0, SCM_EOL);
+  scm_c_define ("<standard-vtable>", scm_standard_vtable_vtable);
 
   scm_applicable_struct_vtable_vtable =
     scm_make_struct (scm_standard_vtable_vtable, SCM_INUM0,
