@@ -1,6 +1,6 @@
 ;;;; ftw.scm --- file system tree walk
 
-;;;; 	Copyright (C) 2002, 2003, 2006 Free Software Foundation, Inc.
+;;;; 	Copyright (C) 2002, 2003, 2006, 2011, 2012 Free Software Foundation, Inc.
 ;;;;
 ;;;; This library is free software; you can redistribute it and/or
 ;;;; modify it under the terms of the GNU Lesser General Public
@@ -190,7 +190,14 @@
 ;;; Code:
 
 (define-module (ice-9 ftw)
-  :export (ftw nftw))
+  #:use-module (ice-9 match)
+  #:use-module (ice-9 vlist)
+  #:use-module (srfi srfi-1)
+  #:autoload   (ice-9 i18n)   (string-locale<?)
+  #:export (ftw nftw
+            file-system-fold
+            file-system-tree
+            scandir))
 
 (define (directory-files dir)
   (let ((dir-stream (opendir dir)))
@@ -377,4 +384,186 @@
         (chdir od)
         ret))))
 
+
+;;;
+;;; `file-system-fold' & co.
+;;;
+
+(define-syntax-rule (errno-if-exception expr)
+  (catch 'system-error
+    (lambda ()
+      expr)
+    (lambda args
+      (system-error-errno args))))
+
+(define* (file-system-fold enter? leaf down up skip error init file-name
+                           #:optional (stat lstat))
+  "Traverse the directory at FILE-NAME, recursively.  Enter
+sub-directories only when (ENTER? PATH STAT RESULT) returns true.  When
+a sub-directory is entered, call (DOWN PATH STAT RESULT), where PATH is
+the path of the sub-directory and STAT the result of (stat PATH); when
+it is left, call (UP PATH STAT RESULT).  For each file in a directory,
+call (LEAF PATH STAT RESULT).  When ENTER? returns false, call (SKIP
+PATH STAT RESULT).  When an `opendir' or STAT call raises an exception,
+call (ERROR PATH STAT ERRNO RESULT), with ERRNO being the operating
+system error number that was raised.
+
+Return the result of these successive applications.
+When FILE-NAME names a flat file, (LEAF PATH STAT INIT) is returned.
+The optional STAT parameter defaults to `lstat'."
+
+  (define (mark v s)
+    (vhash-cons (cons (stat:dev s) (stat:ino s)) #t v))
+
+  (define (visited? v s)
+    (vhash-assoc (cons (stat:dev s) (stat:ino s)) v))
+
+  (let loop ((name     file-name)
+             (path     "")
+             (dir-stat (errno-if-exception (stat file-name)))
+             (result   init)
+             (visited  vlist-null))
+
+    (define full-name
+      (if (string=? path "")
+          name
+          (string-append path "/" name)))
+
+    (cond
+     ((integer? dir-stat)
+      ;; FILE-NAME is not readable.
+      (error full-name #f dir-stat result))
+     ((visited? visited dir-stat)
+      (values result visited))
+     ((eq? 'directory (stat:type dir-stat)) ; true except perhaps the 1st time
+      (if (enter? full-name dir-stat result)
+          (let ((dir     (errno-if-exception (opendir full-name)))
+                (visited (mark visited dir-stat)))
+            (cond
+             ((directory-stream? dir)
+              (let liip ((entry   (readdir dir))
+                         (result  (down full-name dir-stat result))
+                         (subdirs '()))
+                (cond ((eof-object? entry)
+                       (begin
+                         (closedir dir)
+                         (let ((r+v
+                                (fold (lambda (subdir result+visited)
+                                        (call-with-values
+                                            (lambda ()
+                                              (loop (car subdir)
+                                                    full-name
+                                                    (cdr subdir)
+                                                    (car result+visited)
+                                                    (cdr result+visited)))
+                                          cons))
+                                      (cons result visited)
+                                      subdirs)))
+                           (values (up full-name dir-stat (car r+v))
+                                   (cdr r+v)))))
+                      ((or (string=? entry ".")
+                           (string=? entry ".."))
+                       (liip (readdir dir)
+                             result
+                             subdirs))
+                      (else
+                       (let* ((child (string-append full-name "/" entry))
+                              (st    (errno-if-exception (stat child))))
+                         (if (integer? st) ; CHILD is a dangling symlink?
+                             (liip (readdir dir)
+                                   (error child #f st result)
+                                   subdirs)
+                             (if (eq? (stat:type st) 'directory)
+                                 (liip (readdir dir)
+                                       result
+                                       (alist-cons entry st subdirs))
+                                 (liip (readdir dir)
+                                       (leaf child st result)
+                                       subdirs))))))))
+             (else
+              ;; Directory FULL-NAME not readable, but it is stat'able.
+              (values (error full-name dir-stat dir result)
+                      visited))))
+          (values (skip full-name dir-stat result)
+                  (mark visited dir-stat))))
+     (else
+      ;; Caller passed a FILE-NAME that names a flat file, not a directory.
+      (leaf full-name dir-stat result)))))
+
+(define* (file-system-tree file-name
+                           #:optional (enter? (lambda (n s) #t))
+                                      (stat lstat))
+  "Return a tree of the form (FILE-NAME STAT CHILDREN ...) where STAT is
+the result of (STAT FILE-NAME) and CHILDREN are similar structures for
+each file contained in FILE-NAME when it designates a directory.  The
+optional ENTER? predicate is invoked as (ENTER? NAME STAT) and should
+return true to allow recursion into directory NAME; the default value is
+a procedure that always returns #t.  When a directory does not match
+ENTER?, it nonetheless appears in the resulting tree, only with zero
+children.  The optional STAT parameter defaults to `lstat'.  Return #f
+when FILE-NAME is not readable."
+  (define (enter?* name stat result)
+    (enter? name stat))
+  (define (leaf name stat result)
+    (match result
+      (((siblings ...) rest ...)
+       (cons (alist-cons (basename name) (cons stat '()) siblings)
+             rest))))
+  (define (down name stat result)
+    (cons '() result))
+  (define (up name stat result)
+    (match result
+      (((children ...) (siblings ...) rest ...)
+       (cons (alist-cons (basename name) (cons stat children)
+                         siblings)
+             rest))))
+  (define skip                   ; keep an entry for skipped directories
+    leaf)
+  (define (error name stat errno result)
+    (if (string=? name file-name)
+        result
+        (leaf name stat result)))
+
+  (match (file-system-fold enter?* leaf down up skip error '(())
+                           file-name stat)
+    (((tree)) tree)
+    ((())     #f)))                            ; FILE-NAME is unreadable
+
+(define* (scandir name #:optional (select? (const #t))
+                                  (entry<? string-locale<?))
+  "Return the list of the names of files contained in directory NAME
+that match predicate SELECT? (by default, all files.)  The returned list
+of file names is sorted according to ENTRY<?, which defaults to
+`string-locale<?'.  Return #f when NAME is unreadable or is not a directory."
+  (define (enter? dir stat result)
+    (and stat (string=? dir name)))
+
+  (define (visit basename result)
+    (if (select? basename)
+        (cons basename result)
+        result))
+
+  (define (leaf name stat result)
+    (and result
+         (visit (basename name) result)))
+
+  (define (down name stat result)
+    (visit "." '()))
+
+  (define (up name stat result)
+    (visit ".." result))
+
+  (define (skip name stat result)
+    ;; All the sub-directories are skipped.
+    (visit (basename name) result))
+
+  (define (error name* stat errno result)
+    (if (string=? name name*)             ; top-level NAME is unreadable
+        result
+        (visit (basename name*) result)))
+
+  (and=> (file-system-fold enter? leaf down up skip error #f name stat)
+         (lambda (files)
+           (sort files entry<?))))
+
 ;;; ftw.scm ends here
